/* 
 * uuid.c --
 *
 *	Defines the uuid command which creates a DCE UUID and returns the string
 *  form.
 *
 * Copyright 1998, Nat Pryce
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, 
 * SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF 
 * THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE AUTHOR HAS BEEN ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE AUTHOR SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" 
 * BASIS, AND THE AUTHOR HAS NO OBLIGATION TO PROVIDE  MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

#include <string.h>
#include <windows.h>
#include <rpcdce.h>
#include <tcl.h>
#include "uuid.h"


/*
 * Declarations for functions defined in this file.
 */

static int		UuidCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int argc, Tcl_Obj *CONST argv[]));


/*
 *----------------------------------------------------------------------
 *
 * DllEntryPoint --
 *
 *	This wrapper function is used by Windows to invoke the
 *	initialization code for the DLL.  If we are compiling
 *	with Visual C++, this routine will be renamed to DllMain.
 *	routine.
 *
 * Results:
 *	Returns TRUE;
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

#ifdef __WIN32__
BOOL APIENTRY
DllEntryPoint(hInst, reason, reserved)
    HINSTANCE hInst;		/* Library instance handle. */
    DWORD reason;			/* Reason this function is being called. */
    LPVOID reserved;		/* Not used. */
{
	return TRUE;
}
#endif

/*
 *----------------------------------------------------------------------
 *
 * Uuid_Init --
 *
 *	This procedure initializes the uuid command.
 *
 * Results:
 *	A standard Tcl result.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

EXPORT(int,Uuid_Init)( Tcl_Interp *interp ) {
    Tcl_CreateObjCommand( interp, "uuid", UuidCmd, NULL, NULL );
    return Tcl_PkgProvide(interp, "uuid", "1.0");
}

/*
 *----------------------------------------------------------------------
 *
 * Uuid_SafeInit --
 *
 *	This procedure initializes the example command for a safe 
 *      interpreter.  You would leave out of this procedure any commands
 *      you deemed unsafe.
 *
 * Results:
 *	A standard Tcl result.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

EXPORT(int,Uuid_SafeInit)( Tcl_Interp *interp ) {
    return Uuid_Init(interp);
}

/*
 *----------------------------------------------------------------------
 *
 * UuidCmd --
 *
 *	This function implements the Tcl "uuid" command.
 *
 *  This command allocates a universally unique identifier and returns
 *  the string value to the caller.
 *
 * Results:
 *	A standard Tcl result.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

static int 
ErrorString( Tcl_Interp *interp, char *str, RPC_STATUS err ) {
	char err_str[DCE_C_ERROR_STRING_LEN];
	Tcl_Obj *msg;

	DceErrorInqText( err, (unsigned char*)err_str );
	
	msg = Tcl_NewStringObj( str, -1 );
	Tcl_AppendToObj( msg, ": ", -1 );
	Tcl_AppendToObj( msg, err_str, -1 );
	
	Tcl_SetObjResult( interp, msg );
	return TCL_ERROR;
}


static int
UuidCmd( ClientData clientData, Tcl_Interp *interp, 
		 int objc, Tcl_Obj *CONST objv[] )
{
	UUID uuid;
	RPC_STATUS status;
	unsigned char *uuid_str;
	
	if( objc ==1 ) {
		status = UuidCreate(&uuid);
	} else if( objc == 2 ) {
		int len;
		char *str = Tcl_GetStringFromObj( objv[1], &len );

		if( strncmp( "-nil", str, len ) == 0 ) {
			status = UuidCreateNil(&uuid);		
		} else {
			Tcl_Obj *msg = Tcl_NewStringObj( "unknown option \"", -1 );
			Tcl_AppendToObj( msg, str, len );
			Tcl_AppendToObj( msg, "\"", 1 );

			Tcl_SetObjResult( interp, msg );
			return TCL_ERROR;
		}

	} else {
		Tcl_WrongNumArgs( interp, objc, objv, "?-nil?" );
		return TCL_ERROR;
	}

	if( status != RPC_S_OK ) {
		return ErrorString( interp, "failed to create uuid", status );
	}
	
	status = UuidToString( &uuid, &uuid_str );
	if( status != RPC_S_OK ) {
		return ErrorString( interp, "failed to convert uuid to string", status );
	}
	
	Tcl_SetObjResult( interp, Tcl_NewStringObj( uuid_str, -1 ) );
	RpcStringFree(&uuid_str);
	return TCL_OK;
}
